@@ -713,7 +713,13 @@ main(argc, argv)
     	nomagic = 1;	/* Original didn't have a magic number */
 #endif
 
-    	filelist = fileptr = (char **)malloc(argc*sizeof(char *));
+    	filelist = (char **)malloc(argc*sizeof(char *));
+    	if (filelist == NULL)
+		{
+			fprintf(stderr, "Cannot allocate memory for file list.\n");
+			exit (1);
+		}
+    	fileptr = filelist;
     	*filelist = NULL;
 
     	if((progname = strrchr(argv[0], '/')) != 0)
@@ -1236,7 +1242,12 @@ comprexx(fileptr)
 						exit_code = 1;
 					}
 #ifndef	DOS
-					chown(ofname, infstat.st_uid, infstat.st_gid);	/* Copy ownership */
+					if (chown(ofname, infstat.st_uid, infstat.st_gid))	/* Copy ownership */
+					{
+						fprintf(stderr, "\nchown error (ignored) ");
+						perror(ofname);
+						exit_code = 1;
+					}
 #endif
 #endif
 					remove_ofname = 0;
@@ -1670,7 +1681,8 @@ resetbuf:	;
 				int				e;
 				int				o;
 
-				e = insize-(o = (posbits>>3));
+				o = posbits >> 3;
+				e = o <= insize ? insize - o : 0;
 
 				for (i = 0 ; i < e ; ++i)
 					inbuf[i] = inbuf[i+o];
